@@ -1,7 +1,7 @@
 // qsynthAbout.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,11 +22,11 @@
 #ifndef __qsynthAbout_h
 #define __qsynthAbout_h
 
-#define QSYNTH_TITLE        "qsynth"
+#define QSYNTH_TITLE        "Qsynth"
 #define QSYNTH_SUBTITLE	    "A fluidsynth Qt GUI Interface"
-#define QSYNTH_VERSION      "0.2.2"
+#define QSYNTH_VERSION      "0.2.3"
 #define QSYNTH_WEBSITE      "http://qsynth.sourceforge.net"
-#define QSYNTH_COPYRIGHT    "Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved."
+#define QSYNTH_COPYRIGHT    "Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved."
 
 #endif  // __qsynthAbout_h
 
