@@ -2,7 +2,7 @@
 //
 // ui.h extension file, included from the uic-generated form implementation.
 /****************************************************************************
-   Copyright (C) 2003, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,8 +29,11 @@
 // Kind of constructor.
 void qsynthMessagesForm::init (void)
 {
+#if QT_VERSION >= 0x030200
+    MessagesTextView->setTextFormat(Qt::LogText);
+#endif
     // Initialize default message limit.
-    m_iMessagesLimit = QSYNTH_MESSAGES_MAXLINES;
+    setMessagesLimit(QSYNTH_MESSAGES_MAXLINES);
 }
 
 
@@ -82,6 +85,10 @@ int qsynthMessagesForm::messagesLimit (void)
 void qsynthMessagesForm::setMessagesLimit( int iMessagesLimit )
 {
     m_iMessagesLimit = iMessagesLimit;
+    m_iMessagesHigh  = iMessagesLimit + (iMessagesLimit / 3);
+#if QT_VERSION >= 0x030200
+	MessagesTextView->setMaxLogLines(iMessagesLimit);
+#endif
 }
 
 
@@ -98,10 +105,11 @@ void qsynthMessagesForm::appendMessagesColor( const QString& s, const QString& c
 
 void qsynthMessagesForm::appendMessagesText( const QString& s )
 {
+#if QT_VERSION < 0x030200
     // Check for message line limit...
     if (m_iMessagesLimit > 0) {
         int iParagraphs = MessagesTextView->paragraphs();
-        if (iParagraphs > m_iMessagesLimit) {
+        if (iParagraphs > m_iMessagesHigh) {
             MessagesTextView->setUpdatesEnabled(false);
             while (iParagraphs > m_iMessagesLimit) {
                 MessagesTextView->removeParagraph(0);
@@ -111,6 +119,7 @@ void qsynthMessagesForm::appendMessagesText( const QString& s )
             MessagesTextView->setUpdatesEnabled(true);
         }
     }
+#endif
     MessagesTextView->append(s);
 }
 
