@@ -1,7 +1,7 @@
 // qsynthSetup.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -68,35 +68,53 @@ void qsynthSetup::realize (void)
 
     if (!sMidiDriver.isEmpty())
         ::fluid_settings_setstr(m_pFluidSettings, "midi.driver", (char *) sMidiDriver.latin1());
-    if (iMidiChannels > 0)
-        ::fluid_settings_setint(m_pFluidSettings, "synth.midi-channels", iMidiChannels);
-    if (!sAudioDriver.isEmpty())
-        ::fluid_settings_setstr(m_pFluidSettings, "audio.driver", (char *) sAudioDriver.latin1());
-    if (!sJackName.isEmpty())
-        ::fluid_settings_setstr(m_pFluidSettings, "audio.jack.id", (char *) sJackName.latin1());
+    if (!sMidiDevice.isEmpty()) {
+		QString sMidiKey = "midi.";
+		if (sMidiDriver == "alsa_raw")
+			sMidiKey += "alsa";
+		else
+            sMidiKey += sMidiDriver;
+		sMidiKey += + ".device";
+        ::fluid_settings_setstr(m_pFluidSettings, (char *) sMidiKey.latin1(), (char *) sMidiDevice.latin1());
+	}
     if (!sAlsaName.isEmpty())
         ::fluid_settings_setstr(m_pFluidSettings, "midi.alsa_seq.id", (char *) sAlsaName.latin1());
 
+    if (!sAudioDriver.isEmpty())
+        ::fluid_settings_setstr(m_pFluidSettings, "audio.driver", (char *) sAudioDriver.latin1());
+    if (!sAudioDevice.isEmpty()) {
+		QString sAudioKey = "audio." + sAudioDriver + '.';
+		if (sAudioDriver == "file")
+			sAudioKey += "name";
+		else
+            sAudioKey += "device";
+        ::fluid_settings_setstr(m_pFluidSettings, (char *) sAudioKey.latin1(), (char *) sAudioDevice.latin1());
+	}
+    if (!sJackName.isEmpty())
+        ::fluid_settings_setstr(m_pFluidSettings, "audio.jack.id", (char *) sJackName.latin1());
+
     ::fluid_settings_setint(m_pFluidSettings, "audio.jack.autoconnect", (int) bJackAutoConnect);
     ::fluid_settings_setstr(m_pFluidSettings, "audio.jack.multi", (char *) (bJackMulti ? "yes" : "no"));
     ::fluid_settings_setstr(m_pFluidSettings, "audio.sample-format", (char *) sSampleFormat.latin1());
-
-    if (iAudioChannels > 0)
-        ::fluid_settings_setint(m_pFluidSettings, "synth.audio-channels", iAudioChannels);
-    if (iAudioGroups > 0)
-        ::fluid_settings_setint(m_pFluidSettings, "synth.audio-groups", iAudioGroups);
     if (iAudioBufSize > 0)
         ::fluid_settings_setint(m_pFluidSettings, "audio.period-size", iAudioBufSize);
     if (iAudioBufCount > 0)
         ::fluid_settings_setint(m_pFluidSettings, "audio.periods", iAudioBufCount);
+
+    if (iMidiChannels > 0)
+        ::fluid_settings_setint(m_pFluidSettings, "synth.midi-channels", iMidiChannels);
+    if (iAudioChannels > 0)
+        ::fluid_settings_setint(m_pFluidSettings, "synth.audio-channels", iAudioChannels);
+    if (iAudioGroups > 0)
+        ::fluid_settings_setint(m_pFluidSettings, "synth.audio-groups", iAudioGroups);
     if (fSampleRate > 0.0)
         ::fluid_settings_setnum(m_pFluidSettings, "synth.sample-rate", fSampleRate);
     if (iPolyphony > 0)
         ::fluid_settings_setint(m_pFluidSettings, "synth.polyphony", iPolyphony);
+
 //  Gain is set on realtime (don't need to set it here)
 //  if (fGain > 0.0)
 //      ::fluid_settings_setnum(m_pFluidSettings, "synth.gain", fGain);
-
     ::fluid_settings_setstr(m_pFluidSettings, "synth.reverb.active", (char *) (bReverbActive ? "yes" : "no"));
     ::fluid_settings_setstr(m_pFluidSettings, "synth.chorus.active", (char *) (bChorusActive ? "yes" : "no"));
     ::fluid_settings_setstr(m_pFluidSettings, "synth.ladspa.active", (char *) (bLadspaActive ? "yes" : "no"));
