@@ -1,7 +1,7 @@
 // qsynthSetup.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -57,9 +57,11 @@ public:
     // Settings variables.
     bool    bMidiIn;
     QString sMidiDriver;
+    QString sMidiDevice;
     int     iMidiChannels;
     QString sAlsaName;
     QString sAudioDriver;
+    QString sAudioDevice;
     QString sJackName;
     bool    bJackAutoConnect;
     bool    bJackMulti;
